@@ -144,7 +144,7 @@ GST_PAD_TEMPLATE_FACTORY (src_audio_templ,
   ),
   GST_CAPS_NEW (
     "avidemux_src_audio",
-    "audio/mp3",
+    "audio/x-mp3",
       NULL
   ),
   GST_CAPS_NEW (
@@ -169,9 +169,9 @@ static const GstEventMask*
 			gst_avi_demux_get_event_mask 		(GstPad *pad);
 static gboolean 	gst_avi_demux_handle_src_event 		(GstPad *pad, GstEvent *event);
 static const GstFormat* gst_avi_demux_get_src_formats 		(GstPad *pad); 
-static const GstPadQueryType*
+static const GstQueryType*
 			gst_avi_demux_get_src_query_types 	(GstPad *pad);
-static gboolean 	gst_avi_demux_handle_src_query 		(GstPad *pad, GstPadQueryType type, 
+static gboolean 	gst_avi_demux_handle_src_query 		(GstPad *pad, GstQueryType type, 
 								 GstFormat *format, gint64 *value);
 static gboolean 	gst_avi_demux_src_convert 		(GstPad *pad, GstFormat src_format, gint64 src_value,
 	        	          				 GstFormat *dest_format, gint64 *dest_value);
@@ -369,6 +369,7 @@ gst_avi_demux_strh (GstAviDemux *avi_demux)
     target->delay = 0LL;
     target->total_bytes = 0LL;
     target->total_frames = 0;
+    target->end_pos = -1;
 
     target->skip = 0;
 
@@ -519,7 +520,7 @@ gst_avi_demux_strf_auds (GstAviDemux *avi_demux)
     case GST_RIFF_WAVE_FORMAT_MPEGL3:
     case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp3 */
       newcaps = gst_caps_new ("avidemux_audio_src",
-                              "audio/mp3",
+                              "audio/x-mp3",
                                 NULL);
       break;
     case GST_RIFF_WAVE_FORMAT_PCM: /* PCM/wav */
@@ -871,12 +872,12 @@ gst_avi_demux_src_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
   return res;
 }
 
-static const GstPadQueryType*
+static const GstQueryType*
 gst_avi_demux_get_src_query_types (GstPad *pad) 
 {
-  static const GstPadQueryType src_types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType src_types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
     0
   };
 
@@ -884,7 +885,7 @@ gst_avi_demux_get_src_query_types (GstPad *pad)
 }
 
 static gboolean
-gst_avi_demux_handle_src_query (GstPad *pad, GstPadQueryType type, 
+gst_avi_demux_handle_src_query (GstPad *pad, GstQueryType type, 
 				GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
@@ -892,7 +893,7 @@ gst_avi_demux_handle_src_query (GstPad *pad, GstPadQueryType type,
   avi_stream_context *stream = gst_pad_get_element_private (pad);
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
       switch (*format) {
         case GST_FORMAT_DEFAULT:
           *format = GST_FORMAT_TIME;
@@ -920,7 +921,7 @@ gst_avi_demux_handle_src_query (GstPad *pad, GstPadQueryType type,
 	  break;
       }
       break;
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       switch (*format) {
         case GST_FORMAT_DEFAULT:
           *format = GST_FORMAT_TIME;
@@ -1032,6 +1033,7 @@ gst_avi_demux_get_event_mask (GstPad *pad)
 {
   static const GstEventMask masks[] = {
     { GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_KEY_UNIT },
+    { GST_EVENT_SEEK_SEGMENT, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_KEY_UNIT },
     { 0, }
   };
 
@@ -1048,6 +1050,8 @@ gst_avi_demux_handle_src_event (GstPad *pad, GstEvent *event)
   stream = gst_pad_get_element_private (pad);
 
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK_SEGMENT:
+      stream->end_pos = GST_EVENT_SEEK_ENDOFFSET (event);
     case GST_EVENT_SEEK:
       GST_DEBUG (0, "seek format %d, %08x", GST_EVENT_SEEK_FORMAT (event), stream->strh.type);
       switch (GST_EVENT_SEEK_FORMAT (event)) {
@@ -1060,12 +1064,14 @@ gst_avi_demux_handle_src_event (GstPad *pad, GstEvent *event)
 	  gint64 desired_offset = GST_EVENT_SEEK_OFFSET (event);
 	  guint32 flags;
           guint64 min_index;
+	  
 
 	  /* no seek on audio yet */
 	  if (stream->strh.type == GST_RIFF_FCC_auds) {
 	    res = FALSE;
 	    goto done;
 	  }
+          GST_DEBUG (0, "seeking to %lld", desired_offset);
 
           flags = GST_RIFF_IF_KEYFRAME;
 
@@ -1335,7 +1341,7 @@ gst_avi_demux_process_chunk (GstAviDemux *avi_demux, guint64 *filepos,
 		    chunkid, *chunksize, stream_id);
 
       format = GST_FORMAT_TIME;
-      gst_pad_query (stream->pad, GST_PAD_QUERY_POSITION, &format, &next_ts);
+      gst_pad_query (stream->pad, GST_QUERY_POSITION, &format, &next_ts);
 
       if (stream->strh.init_frames == stream->current_frame && stream->delay==0)
 	stream->delay = next_ts;
@@ -1344,27 +1350,34 @@ gst_avi_demux_process_chunk (GstAviDemux *avi_demux, guint64 *filepos,
       stream->current_byte += *chunksize;
 
       if (stream->skip) {
-	stream->skip--;
+        stream->skip--;
       }
       else {
-        if (stream->pad && GST_PAD_IS_CONNECTED (stream->pad)) {
-	  GstBuffer *buf;
-          guint32   got_bytes;
-
-	  if (*chunksize) {
-            got_bytes = gst_bytestream_peek (bs, &buf, *chunksize);
-
-            GST_BUFFER_TIMESTAMP (buf) = next_ts;
-
-            if (stream->need_flush) {
-               /* FIXME, do some flush event here */
-              stream->need_flush = FALSE;
-            }
-	    GST_DEBUG (0, "send stream %d: %lld %d %lld %08x", stream_id, next_ts, stream->current_frame - 1,
-			  stream->delay, *chunksize);
-
-            gst_pad_push(stream->pad, buf);
+        if (GST_PAD_IS_USABLE (stream->pad)) {
+          if (next_ts >= stream->end_pos) {
+	    gst_pad_push (stream->pad, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
+	    GST_DEBUG (0, "end stream %d: %lld %d %lld", stream_id, next_ts, stream->current_frame - 1,
+			    stream->end_pos);
 	  }
+	  else {
+	    GstBuffer *buf;
+            guint32   got_bytes;
+
+	    if (*chunksize) {
+              got_bytes = gst_bytestream_peek (bs, &buf, *chunksize);
+
+              GST_BUFFER_TIMESTAMP (buf) = next_ts;
+
+              if (stream->need_flush) {
+                 /* FIXME, do some flush event here */
+                stream->need_flush = FALSE;
+              }
+	      GST_DEBUG (0, "send stream %d: %lld %d %lld %08x", stream_id, next_ts, stream->current_frame - 1,
+			    stream->delay, *chunksize);
+
+              gst_pad_push(stream->pad, buf);
+	    }
+          }
         }
       }
 
